from typing import Optional

import aiohttp

from weatherflow4py.exceptions import TokenError
from weatherflow4py.models.rest.device import DeviceObservationTempestREST
from weatherflow4py.models.rest.forecast import WeatherDataForecastREST
from weatherflow4py.models.rest.observation import ObservationStationREST
from weatherflow4py.models.rest.stations import StationsResponseREST
from weatherflow4py.models.rest.unified import WeatherFlowDataREST
from .const import REST_LOGGER

from yarl import URL


class WeatherFlowRestAPI:
    """Our REST rate limits are not connected to our web socket rate limits. For REST you can make 100 requests per
    minute. There is some burst capacity built into the system, but the general rule of thumb it to keep the number
    of REST requests per user to under 100 per minute."""

    BASE_URL = "https://swd.weatherflow.com/swd/rest"

    def __init__(self, api_token: str, session: Optional[aiohttp.ClientSession] = None):
        if not api_token:
            raise TokenError

        REST_LOGGER.debug(f"Initializing the WeatherFlow API with token {api_token}")
        self.api_token = api_token
        self._session = session
        self._owned_session = None

    @property
    def session(self):
        if self._session is None:
            if self._owned_session is None:
                self._owned_session = aiohttp.ClientSession(
                    headers={"Accept": "application/json"}
                )
            return self._owned_session
        return self._session

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        # Do not close the session here
        pass

    async def _make_request(
        self, endpoint: str, params: dict = None, response_model=None
    ):
        url = URL(f"{self.BASE_URL}/{endpoint}")
        full_params = {"token": self.api_token, **(params or {})}
        full_url = url.with_query(full_params)

        REST_LOGGER.debug(f"Making request to {full_url}")

        async with self.session.get(url, params=full_params) as response:
            response.raise_for_status()
            data = await response.text()

            REST_LOGGER.debug(f"Received response: {data}")

        try:
            return response_model.from_json(data) if response_model else None
        except Exception as e:
            error_msg = f"Unable to convert data || {data} || to || {response_model} -- {str(e)}"
            print(error_msg)
            REST_LOGGER.error(error_msg)
            raise e

    async def async_get_stations(self) -> StationsResponseREST:
        """
        Gets station_id data.

        Raises:
            ClientResponseError: If there is a client response error.
        """
        ret = await self._make_request("stations", response_model=StationsResponseREST)
        return ret

    async def async_get_station(self, station_id: int) -> StationsResponseREST:
        """
        Gets data for a specific station_id.

        Args:
            station_id (int): The ID of the station_id.

        Raises:
            ClientResponseError: If there is a client response error.
        """
        return (
            await self._make_request(
                f"stations/{station_id}", response_model=StationsResponseREST
            )
        ).stations

    async def async_get_forecast(self, station_id: int) -> WeatherDataForecastREST:
        """
        Gets the forecast for a given station_id.

        Args:
            station_id (int): The ID of the station_id.

        Raises:
            ClientResponseError: If there is a client response error.
        """
        return await self._make_request(
            "better_forecast",
            params={"station_id": station_id},
            response_model=WeatherDataForecastREST,
        )

    async def async_get_device_observations(
        self, device_id: int
    ) -> DeviceObservationTempestREST:
        """
        Gets the device_id observation data for a given device_id.

        Args:
            device_id (int): The ID of the device_id.

        Raises:
            ClientResponseError: If there is a client response error.
        """
        obs_data = await self._make_request(
            f"observations/device/{device_id}",
            response_model=DeviceObservationTempestREST,
        )

        return obs_data

    async def async_get_observation(self, station_id: int) -> ObservationStationREST:
        """
        Gets the observation data for a given station_id.

        Args:
            station_id (int): The ID of the station_id.

        Raises:
            ClientResponseError: If there is a client response error.
        """
        return await self._make_request(
            f"observations/station/{station_id}",
            response_model=ObservationStationREST,
        )

    async def get_all_data(
        self, get_device_observations: bool = False
    ) -> dict[int, WeatherFlowDataREST]:
        """
        Builds a full data set of stations and forecasts. If get_device_observations is True,
        it also fetches device_id observations for each station_id. Otherwise, device_observations
        will be set to None for each station_id.

        Args:
            get_device_observations (bool): Whether to fetch device_id observations for each station_id.

        Returns:
            dict[int, WeatherFlowDataREST]: A dictionary mapping station_id IDs to their corresponding data.

        Raises:
            ClientResponseError: If there is a client response error during data retrieval.
        """
        ret: dict[int, WeatherFlowDataREST] = {}
        station_response = await self.async_get_stations()
        for station in station_response.stations:
            device_id = station.outdoor_devices[0].device_id

            device_observations = None
            if get_device_observations:
                device_observations = await self.async_get_device_observations(
                    device_id=device_id
                )

            ret[station.station_id] = WeatherFlowDataREST(
                weather=await self.async_get_forecast(station_id=station.station_id),
                observation=await self.async_get_observation(
                    station_id=station.station_id
                ),
                station=station,
                device_observations=device_observations,
            )

        return ret

    @classmethod
    async def create(
        cls, api_token: str, session: Optional[aiohttp.ClientSession] = None
    ):
        return cls(api_token, session)

    async def close(self):
        if self._owned_session:
            await self._owned_session.close()
            self._owned_session = None



"""Base message types for websockets."""
from dataclasses import dataclass
from typing import List, Union, Any, Type

from weatherflow4py.models.ws.custom_types import (
    PrecipitationAnalysisType,
    ObservationType,
    PrecipitationType,
)


@dataclass
class base_obs:
    """Base observation class with a class method for creating instances from lists."""

    @classmethod
    def from_list(cls: Type["base_obs"], lst: List) -> "base_obs":
        return cls(*lst)


# Define the base observation classes with a class method for creating instances from lists
@dataclass
class obs_sky(base_obs):
    """Dataclass for the sky observation type."""

    epoch: int
    illuminance: int
    uv: float
    rain_accumulation: float
    wind_lull: float
    wind_avg: float
    wind_gust: float
    wind_direction: int
    battery: float
    report_interval: int
    solar_radiation: float
    local_day_rain_accumulation: float
    precipitation_type: PrecipitationType
    wind_sample_interval: int
    nc_rain: float
    local_day_nc_rain_accumulation: float
    precipitation_analysis_type: PrecipitationAnalysisType


@dataclass
class obs_st(base_obs):
    """Dataclass for the station_id observation type."""

    epoch: int
    wind_lull: float
    wind_avg: float
    wind_gust: float
    wind_direction: int
    wind_sample_interval: int
    pressure: float
    air_temperature: float
    relative_humidity: int
    illuminance: int
    uv: float
    solar_radiation: float
    rain_accumulation: float
    precipitation_type: PrecipitationType
    average_strike_distance: int
    strike_count: int
    battery: float
    report_interval: int
    local_day_rain_accumulation: float
    nc_rain_accumulation: float
    local_day_nc_rain_accumulation: float
    precipitation_analysis_type: PrecipitationAnalysisType

    def __post_init__(self):
        # Transform the raw observation data into the correct instances
        self.precipitation_type = PrecipitationType(self.precipitation_type)
        self.precipitation_analysis_type = PrecipitationAnalysisType(
            self.precipitation_analysis_type
        )


@dataclass
class obs_air(base_obs):
    """Dataclass for the air observation type."""

    epoch: int
    station_pressure: float
    air_temperature: float
    relative_humidity: int
    lightning_strike_count: int
    lightning_strike_average_distance: float
    battery: float
    report_interval: int


class ObservationFactory:
    """Factory class for creating observation instances from lists."""

    @staticmethod
    def create_observation(
        obs_type: ObservationType, obs_data: List[List[Any]]
    ) -> List[Union[obs_sky, obs_st, obs_air]]:
        """Create observation instances from a list of lists."""

        # if type(obs_data[0]) != list:
        #     print("OBS Data is already correct - aborting")
        #     return obs_data

        obs_class_map = {
            ObservationType.OBS_SKY: obs_sky,
            ObservationType.OBS_ST: obs_st,
            ObservationType.OBS_AIR: obs_air,
        }
        obs_class = obs_class_map.get(obs_type)
        if not obs_class:
            raise ValueError(f"Unknown observation type: {obs_type}")
        return [obs_class.from_list(obs_item) for obs_item in obs_data]


@dataclass
class WebsocketObservation:
    """Dataclass for the websocket observation type."""

    type: ObservationType
    device_id: int
    obs: List[Union[obs_sky, obs_st, obs_air]]

    def __post_init__(self):
        # Transform the raw observation data into the correct instances
        self.type = ObservationType(self.type)

        # TODO: Figure out why sometimes this gets called twice and why we need the logic
        # if not isinstance(self.obs[0], (obs_sky, obs_st, obs_air)):
        self.obs = ObservationFactory.create_observation(self.type, self.obs)

    #
    # @classmethod
    # def from_json(cls, json_data: dict[str, Any]) -> "WebsocketObservation":
    #     """Create a WebsocketObservation instance from a JSON dictionary."""
    #     obs_type = ObservationType(json_data["type"])
    #     obs_data: list[obs_sky | obs_air | obs_st] = json_data["obs"]
    #     observation_instances = ObservationFactory.create_observation(
    #         obs_type, obs_data
    #     )
    #     return cls(
    #         type=obs_type, device_id=json_data["device_id"], obs=observation_instances
    #     )
    #
    # @classmethod
    # def from_dict(cls, data: dict[str, Any]) -> "WebsocketObservation":
    #     """Create a WebsocketObservation instance from a dictionary."""
    #     obs_type = ObservationType(data["type"])
    #     obs_data: list[obs_sky | obs_air | obs_st] = data["obs"]
    #     observation_instances = ObservationFactory.create_observation(
    #         obs_type, obs_data
    #     )
    #     return cls(
    #         type=obs_type, device_id=data["device_id"], obs=observation_instances
    #     )

    @property
    def first(self) -> obs_sky | obs_air | obs_st:
        """Return the first observation instance."""
        return self.obs[0]

    @property
    def epoch(self) -> int:
        return self.first.epoch

    @property
    def wind_lull(self) -> float:
        return self.first.wind_lull

    @property
    def wind_avg(self) -> float:
        return self.first.wind_avg

    @property
    def wind_gust(self) -> float:
        return self.first.wind_gust

    @property
    def wind_direction(self) -> int:
        return self.first.wind_direction

    @property
    def wind_sample_interval(self) -> int:
        return self.first.wind_sample_interval

    @property
    def pressure(self) -> float:
        return self.first.pressure

    @property
    def air_temperature(self) -> float:
        return self.first.air_temperature

    @property
    def relative_humidity(self) -> int:
        return self.first.relative_humidity

    @property
    def illuminance(self) -> int:
        return self.first.illuminance

    @property
    def uv(self) -> float:
        return self.first.uv

    @property
    def solar_radiation(self) -> float:
        return self.first.solar_radiation

    @property
    def rain_accumulation(self) -> float:
        return self.first.rain_accumulation

    @property
    def precipitation_type(self) -> PrecipitationType:
        return self.first.precipitation_type

    @property
    def average_strike_distance(self) -> int:
        return self.first.average_strike_distance

    @property
    def strike_count(self) -> int:
        return self.first.strike_count

    @property
    def battery(self) -> float:
        return self.first.battery

    @property
    def report_interval(self) -> int:
        return self.first.report_interval

    @property
    def local_day_rain_accumulation(self) -> float:
        return self.first.local_day_rain_accumulation

    @property
    def nc_rain_accumulation(self) -> float:
        return self.first.nc_rain_accumulation

    @property
    def local_day_nc_rain_accumulation(self) -> float:
        return self.first.local_day_nc_rain_accumulation

    @property
    def precipitation_analysis_type(self) -> PrecipitationAnalysisType:
        return self.first.precipitation_analysis_type


# def construct_obs_st(sky: obs_sky, air: obs_air) -> obs_st:
#     return obs_st(
#         epoch=sky.epoch,  # Assuming the epochs are the same
#         wind_lull=sky.wind_lull,
#         wind_avg=sky.wind_avg,
#         wind_gust=sky.wind_gust,
#         wind_direction=sky.wind_direction,
#         wind_sample_interval=sky.wind_sample_interval,
#         pressure=air.station_pressure,
#         air_temperature=air.air_temperature,
#         relative_humidity=air.relative_humidity,
#         illuminance=sky.illuminance,
#         uv=sky.uv,
#         solar_radiation=sky.solar_radiation,
#         rain_accumulation=sky.rain_accumulation,
#         precipitation_type=sky.precipitation_type,
#         average_strike_distance=air.lightning_strike_average_distance,
#         strike_count=air.lightning_strike_count,
#         battery=sky.battery,  # Assuming the battery levels are the same
#         report_interval=sky.report_interval,
#         local_day_rain_accumulation=sky.local_day_rain_accumulation,
#         nc_rain_accumulation=sky.nc_rain,  # Assuming nc_rain is the same as nc_rain_accumulation
#         local_day_nc_rain_accumulation=sky.local_day_nc_rain_accumulation,
#         precipitation_analysis_type=sky.precipitation_analysis_type
#     )



from __future__ import annotations

from dataclasses import dataclass


from weatherflow4py.models.rest.device import DeviceObservationTempestREST
from weatherflow4py.models.rest.forecast import WeatherDataForecastREST
from weatherflow4py.models.rest.observation import ObservationStationREST
from weatherflow4py.models.rest.stations import Stations


@dataclass
class WeatherFlowDataREST:
    """Unified Weather Data Object.

    API suggests to only use station_id Observations in most cases - so we can keep device_observations in general as None
    https://apidocs.tempestwx.com/reference/station-vs-device#should-i-use-a-device-or-a-station-observation
    """

    weather: WeatherDataForecastREST
    station: Stations
    observation: ObservationStationREST
    device_observations: DeviceObservationTempestREST | None

    @property
    def primary_device_id(self) -> int:
        return self.station.outdoor_devices[0].device_id



import time
from collections.abc import Callable

import asyncio
from ssl import SSLContext
from typing import Optional

import websockets
import json

from weatherflow4py.models.ws.types import EventType
from weatherflow4py.models.ws.websocket_request import (
    WebsocketRequest,
    ListenStopMessage,
    RapidWindListenStopMessage,
)
from weatherflow4py.models.ws.websocket_response import (
    WebsocketResponseBuilder,
    ObservationTempestWS,
    RapidWindWS,
)

from .const import WS_LOGGER


class WeatherFlowWebsocketAPI:
    """Websocket API For Weatherflow Devices."""

    def __init__(self, access_token: str, device_ids=None):
        if device_ids is None:
            device_ids = []
        self.device_ids = device_ids
        self.uri = f"wss://ws.weatherflow.com/swd/data?token={access_token}"
        self.websocket = None
        self.messages = {}
        self.is_listening = False
        self.listen_task = None  # To keep track of the listening task
        self.callbacks = {}

        WS_LOGGER.debug("WebsocketAPI initialized with URI: " + self.uri)

    def register_callback(
        self, message_type: EventType, callback: Callable[[str], None]
    ):
        """Register a callback for a specific message type"""
        self.callbacks[message_type.value] = callback

    def register_invalid_data_callback(self, callback: Callable[[str], None]):
        """
        Register a callback for the 'invalid' event.

        The callback should be a function that takes a single argument of type str.

        Example:
            def invalid_callback(data: str):
                print("Received invalid data:", data)

            api = WebsocketAPI(device_id, access_token)
            api.register_invalid(invalid_callback)

        Args:
            callback (Callable[[str], None]): The callback function to register.
        """
        self.callbacks[EventType.INVALID.value] = callback

    def register_wind_callback(self, callback: Callable[[RapidWindWS], None]):
        """
        Register a callback for the 'rapid_wind' event.

        The callback should be a function that takes a single argument of type RapidWindWS.

        Example:
            def wind_callback(data: RapidWindWS):
                print("Received wind data:", data)

            api = WebsocketAPI(access_token, device_id )
            api.register_wind_callback(wind_callback)

        Args:
            callback (Callable[[RapidWindWS], None]): The callback function to register.
        """
        self.callbacks[EventType.RAPID_WIND.value] = callback

    def register_precipitation_callback(self, callback: Callable[[str], None]):
        """
        Register a callback for the 'rain' event.

        The callback should be a function that takes a single argument of type str.

        Example:
            def rain_callback(data: str):
                print("Received rain data:", data)

            api = WebsocketAPI(access_token, [device_id1, device_id2])
            api.register_precipitation_callback(rain_callback)

        Args:
            callback (Callable[[str], None]): The callback function to register.
        """
        self.callbacks[EventType.RAIN.value] = callback

    def register_lightning_callback(self, callback: Callable[[str], None]):
        """
        Register a callback for the 'lightning_strike' event.

        The callback should be a function that takes a single argument of type str.

        Example:
            def lightning_callback(data: str):
                print("Received lightning data:", data)

            api = WebsocketAPI(access_token, [device_id1, device_id2])
            api.register_lightning_callback(lightning_callback)

        Args:
            callback (Callable[[str], None]): The callback function to register.
        """
        self.callbacks[EventType.LIGHTNING_STRIKE.value] = callback

    def register_observation_callback(
        self, callback: Callable[[ObservationTempestWS], None]
    ):
        """
        Register a callback for the 'obs_st' event.

        The callback should be a function that takes a single argument of type ObservationTempestWS.

        Example:
            def observation_callback(data: ObservationTempestWS):
                print("Received observation data:", data)

            api = WebsocketAPI(access_token)
            api.register_observation_callback(observation_callback)

        Args:
            callback (Callable[[ObservationTempestWS], None]): The callback function to register.
        """
        self.callbacks[EventType.OBSERVATION.value] = callback

    @property
    def last_observation(self) -> ObservationTempestWS | None:
        """Last observation"""
        return self.messages.get("obs_st")

    def last_wind(self) -> RapidWindWS | None:
        return self.messages.get("rapid_wind")

    def last_observation_time(self) -> float | None:
        """Seconds since last observation"""
        current_epoch = time.time()
        if obs := self.last_observation:
            last_observation_epoch = obs.epoch
            time_difference = current_epoch - last_observation_epoch
            return time_difference
        return None

    async def send_message(self, message_type: WebsocketRequest):
        message = message_type.json
        WS_LOGGER.debug(f"Sending message: {message}")
        await self._send(message)

    async def connect(self, ssl_context: Optional[SSLContext] = None):
        """Establishes a WebSocket connection and starts a background listening task.

        :param ssl_context: Optional SSL context for secure connections
        """
        if ssl_context is None:
            self.websocket = await websockets.connect(self.uri)
        else:
            self.websocket = await websockets.connect(self.uri, ssl=ssl_context)

        # Run the listen method in the background and name the task for easier debugging
        self.listen_task = asyncio.create_task(
            self.listen(), name="WebSocketListenTask"
        )

    async def listen(self):
        self.is_listening = True
        try:
            async for message in self.websocket:
                WS_LOGGER.debug(f"Received message: {message}")
                data = json.loads(message)
                try:
                    response = WebsocketResponseBuilder.build_response(data)
                    self.messages[data["type"]] = response

                    if data["type"] in self.callbacks:
                        if asyncio.iscoroutinefunction(self.callbacks[data["type"]]):
                            WS_LOGGER.debug(
                                f"Calling ASYNC callback for message type: {data['type']}"
                            )
                            # If it is, use 'await' to call it
                            await self.callbacks[data["type"]](response)
                        else:
                            WS_LOGGER.debug(
                                f"Calling SYNC callback for message type: {data['type']}"
                            )
                            # If it's not, call it normally
                            self.callbacks[data["type"]](response)
                    else:
                        WS_LOGGER.debug(f"NO CALLBACK for message type: {data['type']}")
                except ValueError:
                    if EventType.INVALID.value in self.callbacks:
                        if asyncio.iscoroutinefunction(
                            self.callbacks[EventType.INVALID.value]
                        ):
                            # If it is, use 'await' to call it
                            await self.callbacks[EventType.INVALID.value](data)
                        else:
                            # If it's not, call it normally
                            self.callbacks[EventType.INVALID.value](data)
                    else:
                        WS_LOGGER.warning(f"Unrecognized WS Message: {message}")

                    continue

        finally:
            self.is_listening = False

    async def _send(self, message):
        if self.websocket:
            await self.websocket.send(message)

    def is_connected(self):
        # Check if the websocket connection is open
        return self.websocket and not self.websocket.closed

    async def close(self, timeout: float = 5.0) -> None:
        """
        Close the WebSocket connection and clean up resources.

        Args:
            timeout (float): Maximum time to wait for tasks to complete (default: 5.0 seconds)
        """
        if not self.is_connected:
            return

        # Stop listening for all devices
        stop_tasks = []
        for device_id in self.device_ids:
            stop_tasks.extend(
                [
                    self.send_message(ListenStopMessage(device_id=device_id)),
                    self.send_message(RapidWindListenStopMessage(device_id=device_id)),
                ]
            )

        # Wait for all stop messages to be sent
        if stop_tasks:
            await asyncio.gather(*stop_tasks, return_exceptions=True)

        # Cancel the listen task
        if self.listen_task and not self.listen_task.done():
            self.listen_task.cancel()
            try:
                await asyncio.wait_for(self.listen_task, timeout=timeout)
            except asyncio.TimeoutError:
                WS_LOGGER.warning("Listen task cancellation timed out")
            except asyncio.CancelledError:
                WS_LOGGER.info("Listen task was cancelled")

        # Close the WebSocket connection
        if self.websocket:
            try:
                await asyncio.wait_for(self.websocket.close(), timeout=timeout)
            except TimeoutError:
                WS_LOGGER.warning("WebSocket close operation timed out")
            finally:
                if self.websocket.closed:
                    WS_LOGGER.info("WebSocket connection successfully closed")
                else:
                    WS_LOGGER.warning("WebSocket connection not closed")
                self.websocket = None

        self.is_listening = False
        WS_LOGGER.info("WebSocket connection closed and resources cleaned up")



"""Websocket Responses.

Documentation form:
 - https://apidocs.tempestwx.com/reference/websocket-reference#lightning-strike-event
 - https://weatherflow.github.io/Tempest/api/ws.html
"""
from __future__ import annotations

from dataclasses import dataclass
from typing import List

from dataclasses_json import dataclass_json, Undefined, CatchAll

from weatherflow4py.models.rest.device import Summary
from weatherflow4py.models.rest.forecast import WindDirection
from weatherflow4py.models.ws.obs import WebsocketObservation


@dataclass_json(undefined=Undefined.INCLUDE)
@dataclass
class BaseResponseWS:
    type: str
    unknown_fields: CatchAll

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.__dict__ == other.__dict__


@dataclass_json
@dataclass
class AcknowledgementWS(BaseResponseWS):
    id: str


@dataclass_json
@dataclass
class RainStartEventWS(BaseResponseWS):
    device_id: int


@dataclass
class EventDataRapidWind:
    epoch: int
    wind_speed_meters_per_second: int
    wind_direction_degrees: int
    wind_direction_cardinal: WindDirection | None = None

    def __post_init__(self):
        self.wind_direction_cardinal = self._wind_cardinal_direction()

    def _wind_cardinal_direction(self):
        dirs = [
            WindDirection.N,
            WindDirection.NNE,
            WindDirection.NE,
            WindDirection.ENE,
            WindDirection.E,
            WindDirection.ESE,
            WindDirection.SE,
            WindDirection.SSE,
            WindDirection.S,
            WindDirection.SSW,
            WindDirection.SW,
            WindDirection.WSW,
            WindDirection.W,
            WindDirection.WNW,
            WindDirection.NW,
            WindDirection.NNW,
        ]
        ix = round(self.wind_direction_degrees / (360.0 / len(dirs)))
        return dirs[ix % len(dirs)]

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.__dict__ == other.__dict__


@dataclass
class EventDataLightningStrike:
    epoch: int
    distance_km: int
    energy: int

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.__dict__ == other.__dict__


@dataclass_json
@dataclass
class LightningStrikeEventWS(BaseResponseWS):
    """
    Field   Type	    Description
    0	    timestamp	Epoch (seconds, UTC)
    1	    distance	(km)
    2	    energy
    """

    device_id: int
    evt: EventDataLightningStrike | List

    def __post_init__(self):
        self.evt = EventDataLightningStrike(self.evt[0], self.evt[1], self.evt[2])

    @property
    def epoch(self) -> int:
        return self.evt.epoch

    @property
    def distance_km(self) -> int:
        return self.evt.distance_km

    @property
    def energy(self) -> int:
        return self.evt.energy


@dataclass_json
@dataclass
class RapidWindWS(BaseResponseWS):
    """
    Rapid Wind stuff.

        Index	Field	        Units
        0	    Time Epoch	    Seconds
        1	    Wind Speed	    m/s
        2	    Wind Direction	Degrees
    """

    device_id: int
    ob: EventDataRapidWind | List

    def __post_init__(self):
        self.ob = EventDataRapidWind(self.ob[0], self.ob[1], self.ob[2])


@dataclass_json
@dataclass
class ObservationAirWS(BaseResponseWS, WebsocketObservation):
    device_id: int
    summary: Summary
    source: str
    serial_number: str
    hub_sn: str
    firmware_revision: str


@dataclass_json
@dataclass
class ObservationSkyWS(BaseResponseWS, WebsocketObservation):
    device_id: int
    summary: Summary
    source: str
    serial_number: str
    hub_sn: str
    firmware_revision: str


@dataclass_json
@dataclass
class ObservationTempestWS(BaseResponseWS, WebsocketObservation):
    device_id: int
    summary: Summary
    source: str
    serial_number: str
    hub_sn: str
    firmware_revision: str


@dataclass_json
@dataclass
class ConnectionOpenWS(BaseResponseWS):
    pass


class WebsocketResponseBuilder:
    @staticmethod
    def build_response(data: dict):
        type_class_map = {
            "ack": AcknowledgementWS,
            "evt_precip": RainStartEventWS,
            "evt_strike": LightningStrikeEventWS,
            "rapid_wind": RapidWindWS,
            "obs_air": ObservationAirWS,
            "obs_sky": ObservationSkyWS,
            "obs_st": ObservationTempestWS,
            "connection_opened": ConnectionOpenWS,
        }

        response_type = data.get("type")
        response_class = type_class_map.get(response_type)

        if response_class is None:
            raise ValueError(f"Invalid type: {response_type}")
        try:
            return response_class.from_dict(data)
        except KeyError as exec:
            raise ValueError(f"Invalid response: {data}") from exec



from dataclasses import dataclass

from dataclasses_json import dataclass_json
from weatherflow4py.const import BASE_LOGGER


@dataclass_json
@dataclass
class Status:
    status_code: int
    status_message: str

    def __post_init__(self):
        if (
            self.status_message
            == "SUCCESS - Either no capabilities or no recent observations"
        ):
            BASE_LOGGER.debug(
                "No Capabilities or Recent Observations - Station may be offline"
            )
